
from anthill.common.database import DuplicateError, DatabaseError
from anthill.common.internal import Internal, InternalError
from anthill.common.model import Model
from anthill.common.validate import validate
from anthill.common import clamp

import ujson


class BlogEntryAdapter(object):
    def __init__(self, data):
        self.entry_id = data.get("entry_id")
        self.blog_id = data.get("blog_id")
        self.data = data.get("entry_data")
        self.enabled = data.get("entry_enabled") == 1
        self.create_date = data.get("entry_create_dt")
        self.update_date = data.get("entry_update_dt")

    def dump(self):
        return {
            "id": self.entry_id,
            "data": self.data,
            "create_date": str(self.create_date),
            "update_date": str(self.update_date)
        }


class BlogAdapter(object):
    def __init__(self, data):
        self.blog_id = data.get("blog_id")
        self.name = data.get("blog_name")
        self.schema = data.get("blog_schema")
        self.enabled = data.get("blog_enabled") == 1

    def dump(self):
        return {
            "name": self.name,
            "enabled": self.enabled
        }


class BlogError(Exception):
    def __init__(self, code, reason):
        super(Exception, self).__init__(reason)
        self.code = code


class BlogNotFound(Exception):
    pass


class BlogEntryNotFound(Exception):
    pass


class BlogsModel(Model):
    def __init__(self, db, app):
        self.db = db
        self.app = app
        self.internal = Internal()

    def get_setup_tables(self):
        return ["blogs", "blog_entries"]

    def get_setup_db(self):
        return self.db

    def has_delete_account_event(self):
        return False

    @validate(gamespace_id="int", blog_name="str_name", blog_schema="json_dict", blog_enabled="bool")
    async def create_blog(self, gamespace_id, blog_name, blog_schema, blog_enabled):
        try:
            blog_id = await self.db.insert(
                """
                INSERT INTO `blogs`
                (`gamespace_id`, `blog_name`, `blog_schema`, `blog_enabled`) 
                VALUES (%s, %s, %s, %s);
                """, gamespace_id, blog_name, ujson.dumps(blog_schema), int(blog_enabled))
        except DuplicateError:
            raise BlogError(409, "Blog with name '{0}' already exists".format(blog_name))
        except DatabaseError as e:
            raise BlogError(500, "Failed to create blog: " + str(e.args[1]))

        return blog_id

    @validate(gamespace_id="int", blog_id="int")
    async def delete_blog(self, gamespace_id, blog_id):
        try:
            async with self.db.acquire() as db:
                await db.execute(
                    """
                    DELETE FROM `blog_entries`
                    WHERE `gamespace_id`=%s AND `blog_id`=%s;
                    """, gamespace_id, blog_id)
                await db.execute(
                    """
                    DELETE FROM `blogs`
                    WHERE `gamespace_id`=%s AND `blog_id`=%s;
                    """, gamespace_id, blog_id)
        except DatabaseError as e:
            raise BlogError(500, str(e.args[1]))

    @validate(gamespace_id="int", blog_id="int", blog_name="str_name", blog_schema="json_dict", blog_enabled="bool")
    async def update_blog(self, gamespace_id, blog_id, blog_name, blog_schema, blog_enabled):
        try:
            await self.db.execute(
                """
                UPDATE `blogs`
                SET `blog_name`=%s, `blog_schema`=%s, `blog_enabled`=%s
                WHERE `gamespace_id`=%s AND `blog_id`=%s;
                """,  blog_name, ujson.dumps(blog_schema), int(blog_enabled), gamespace_id, blog_id)
        except DuplicateError:
            raise BlogError(409, "Blog with name '{0}' already exists".format(blog_name))
        except DatabaseError as e:
            raise BlogError(500, "Failed to update blog: " + str(e.args[1]))

    @validate(gamespace_id="int", blog_name="str_name")
    async def find_blog(self, gamespace_id, blog_name, db=None):
        try:
            result = await (db or self.db).get("""
                SELECT *
                FROM `blogs`
                WHERE `blog_name`=%s AND `gamespace_id`=%s
                LIMIT 1;
            """, blog_name, gamespace_id)
        except DatabaseError as e:
            raise BlogError(500, "Failed to find blog: " + e.args[1])
        if result is None:
            raise BlogNotFound()
        return BlogAdapter(result)

    @validate(gamespace_id="int", blog_id="int")
    async def get_blog(self, gamespace_id, blog_id, db=None):
        try:
            result = await (db or self.db).get("""
                SELECT *
                FROM `blogs`
                WHERE `blog_id`=%s AND `gamespace_id`=%s
                LIMIT 1;
            """, blog_id, gamespace_id)
        except DatabaseError as e:
            raise BlogError(500, "Failed to get blog: " + e.args[1])
        if result is None:
            raise BlogNotFound()
        return BlogAdapter(result)

    @validate(gamespace_id="int")
    async def list_blogs(self, gamespace_id):
        result = await self.db.query("""
            SELECT `blog_name`, `blog_id`, `blog_enabled`
            FROM `blogs`
            WHERE `gamespace_id`=%s;
        """, gamespace_id)

        return list(map(BlogAdapter, result))

    @validate(gamespace_id="int", blog_id="int", entry_data="json_dict", entry_enabled="bool")
    async def create_blog_entry(self, gamespace_id, blog_id, entry_data, entry_enabled):
        try:
            entry_id = await self.db.insert(
                """
                INSERT INTO `blog_entries`
                (`gamespace_id`, `blog_id`, `entry_data`, `entry_enabled`) 
                VALUES (%s, %s, %s, %s);
                """, gamespace_id, blog_id, ujson.dumps(entry_data), int(entry_enabled))
        except DatabaseError as e:
            raise BlogError(500, "Failed to create blog entry: " + str(e.args[1]))

        return entry_id

    @validate(gamespace_id="int", blog_id="int", entry_id="int")
    async def delete_blog_entry(self, gamespace_id, blog_id, entry_id):
        try:
            await self.db.execute(
                """
                DELETE FROM `blog_entries`
                WHERE `gamespace_id`=%s AND `blog_id`=%s AND `entry_id`=%s;
                """, gamespace_id, blog_id, entry_id)
        except DatabaseError as e:
            raise BlogError(500, str(e.args[1]))

    @validate(gamespace_id="int", blog_id="int", entry_id="int", entry_data="json_dict", entry_enabled="bool")
    async def update_blog_entry(self, gamespace_id, blog_id, entry_id, entry_data, entry_enabled):
        try:
            await self.db.execute(
                """
                UPDATE `blog_entries`
                SET `entry_data`=%s, `entry_enabled`=%s, `entry_update_dt`=CURRENT_TIMESTAMP()
                WHERE `gamespace_id`=%s AND `blog_id`=%s AND `entry_id`=%s;
                """,ujson.dumps(entry_data), int(entry_enabled), gamespace_id, blog_id, entry_id)
        except DatabaseError as e:
            raise BlogError(500, "Failed to update blog entry: " + str(e.args[1]))

    @validate(gamespace_id="int", blog_id="int")
    async def get_blog_entry(self, gamespace_id, blog_id, entry_id, db=None):
        try:
            result = await (db or self.db).get("""
                SELECT *
                FROM `blog_entries`
                WHERE `blog_id`=%s AND `gamespace_id`=%s AND `entry_id`=%s
                LIMIT 1;
            """, blog_id, gamespace_id, entry_id)
        except DatabaseError as e:
            raise BlogError(500, "Failed to get blog entry: " + e.args[1])
        if result is None:
            raise BlogEntryNotFound()
        return BlogEntryAdapter(result)

    @validate(gamespace_id="int", blog_id="int")
    async def list_enabled_blog_entries(self, gamespace_id, blog_id, limit=20):

        if limit <= 0 or limit > 1000:
            raise BlogError(400, "Bad limit")

        async with self.db.acquire() as db:
            result = await db.query("""
                SELECT `entry_id`, `entry_data`, `entry_create_dt`, `entry_update_dt`
                FROM `blog_entries`
                WHERE `gamespace_id`=%s AND `blog_id`=%s AND `entry_enabled`=1
                ORDER BY `entry_id` DESC
                LIMIT %s;
            """, gamespace_id, blog_id, limit)

        return list(map(BlogEntryAdapter, result))

    @validate(gamespace_id="int", blog_id="int")
    async def list_blog_entries_pages(self, gamespace_id, blog_id, entries_in_page, page=1):

        async with self.db.acquire() as db:
            pages_count = await db.get(
                """
                    SELECT COUNT(*) as `count`
                    FROM `blog_entries`
                    WHERE gamespace_id=%s AND `blog_id`=%s;
                """, gamespace_id, blog_id)

            import math
            pages = int(math.ceil(float(pages_count["count"]) / float(entries_in_page)))

            page = clamp(page, 1, pages)

            limit_a = (page - 1) * entries_in_page
            limit_b = page * entries_in_page

            result = await db.query("""
                SELECT *
                FROM `blog_entries`
                WHERE `gamespace_id`=%s AND `blog_id`=%s
                ORDER BY `entry_id` DESC
                LIMIT %s, %s;
            """, gamespace_id, blog_id, limit_a, limit_b)

        return pages, list(map(BlogEntryAdapter, result))



from anthill.common.validate import validate
from anthill.common import admin as a, update

from .model.blog import BlogNotFound, BlogError

import ujson
import collections


class RootAdminController(a.AdminController):
    async def get(self):
        try:
            blogs = await self.application.blogs.list_blogs(self.gamespace)
        except BlogError as e:
            raise a.ActionError(e.args[0])

        return {
            "blogs": blogs
        }

    def render(self, data):
        tbl_rows = [{
            "edit": [a.link("blog", blog.name, icon="rss", blog_id=blog.blog_id)],
            "enabled": [a.status("Yes" if blog.enabled else "No", "success" if blog.enabled else "danger")],
        } for blog in data["blogs"]]

        return [
            a.content("Blogs", [
                {
                    "id": "edit",
                    "title": "Blog"
                }, {
                    "id": "enabled",
                    "title": "Enabled"
                }], tbl_rows, "default", empty="No blogs"),
            a.links("Actions", [
                a.link("new_blog", "New Blog", icon="plus")
            ])
        ]

    def access_scopes(self):
        return ["blog_admin"]


class NewBlogAdminController(a.AdminController):
    def render(self, data):

        return [
            a.breadcrumbs([], "Create New"),
            a.form("Create New Blog", fields={
                "name": a.field("Blog Name", "text", "primary", "non-empty", order=1),
                "schema": a.field("Blog Schema", "json", "primary", order=2,
                                  description="A JSON Scheme every blog entry will be matched on. "
                                              "See docs below for more information."),
                "enabled": a.field("Enabled", "switch", "primary", order=3)
            }, methods={
                "create": a.method("Create", "primary")
            }, data=data),
            a.links("Navigate", [
                a.link("index", "Go back", icon="chevron-left"),
                a.link("https://spacetelescope.github.io/understanding-json-schema/index.html", "See docs", icon="book")
            ])
        ]

    async def get(self):
        return {
            "schema": {
                "type": "object",
                "properties": {
                    "title": {
                        "title": "Title",
                        "type": "object",
                        "properties": {
                            "en": {
                                "title": "English Title",
                                "type": "string"
                            }
                        }
                    }
                },
                "title": "Blog Entry"
            },
            "enabled": "true"
        }

    @validate(name="str_name", schema="load_json_dict", enabled="bool")
    async def create(self, name, schema, enabled):
        try:
            blog_id = await self.application.blogs.create_blog(self.gamespace, name, schema, enabled)
        except BlogError as e:
            raise a.ActionError(e.args[0])

        raise a.Redirect("blog", message="New blog has been created", blog_id=blog_id)

    def access_scopes(self):
        return ["blog_admin"]


class BlogAdminController(a.AdminController):
    ENTRIES_PER_PAGE = 20

    def render(self, data):

        tbl_rows = [{
            "edit": [a.link(
                "blog_entry",
                entry.data["title"]["en"]
                if entry.data.get("title", {}).get("en", None) is not None
                else entry.entry_id, icon="rss",
                blog_id=self.context.get("blog_id"),
                entry_id=entry.entry_id)],
            "enabled": [a.status("Yes" if entry.enabled else "No", "success" if entry.enabled else "danger")],
            "created": str(entry.create_date),
            "updated": str(entry.update_date)
        } for entry in data["entries"]]

        return [
            a.breadcrumbs([], data["blog_name"]),
            a.content("Entries", [
                {
                    "id": "edit",
                    "title": "Edit"
                }, {
                    "id": "enabled",
                    "title": "Enabled"
                }, {
                    "id": "created",
                    "title": "Created"
                }, {
                    "id": "updated",
                    "title": "Updated"
                }], tbl_rows, "default", empty="No blog entries"),
            a.pages(data["pages"]),
            a.links("Navigate", [
                a.link("index", "Go back", icon="chevron-left"),
                a.link("new_blog_entry", "New Blog Entry", icon="plus", blog_id=self.context.get("blog_id")),
                a.link("blog_settings", "Settings", icon="cogs", blog_id=self.context.get("blog_id"))
            ])
        ]

    @validate(blog_id="int", page="int")
    async def get(self, blog_id, page=1):
        try:
            blog = await self.application.blogs.get_blog(self.gamespace, blog_id)
        except BlogNotFound:
            raise a.ActionError("No such blog")

        try:
            pages, entries = await self.application.blogs.list_blog_entries_pages(
                self.gamespace, blog_id, BlogAdminController.ENTRIES_PER_PAGE, page)
        except BlogError as e:
            raise a.ActionError(e.args[0])

        return {
            "blog_name": blog.name,
            "pages": pages,
            "entries": entries
        }

    def access_scopes(self):
        return ["blog_admin"]


class BlogSettingsAdminController(a.AdminController):
    def render(self, data):

        return [
            a.breadcrumbs([
                a.link("blog", data["name"], blog_id=self.context.get("blog_id"))
            ], "Settings"),
            a.form("Blog Settings", fields={
                "name": a.field("Blog Name", "text", "primary", "non-empty", order=1),
                "schema": a.field("Blog Schema", "json", "primary", order=2,
                                  description="A JSON Scheme every blog entry will be matched on. "
                                              "See docs below for more information."),
                "enabled": a.field("Enabled", "switch", "primary", order=3)
            }, methods={
                "update": a.method("Update", "primary")
            }, data=data),
            a.form("Danger", fields={}, methods={
                "delete": a.method("Delete This Blog Forever", "danger")
            }, data=data, icon="warning"),
            a.links("Navigate", [
                a.link("index", "Go back", icon="chevron-left"),
                a.link("https://spacetelescope.github.io/understanding-json-schema/index.html", "See docs", icon="book")
            ])
        ]

    @validate(name="str_name", schema="load_json_dict", enabled="bool")
    async def update(self, name, schema, enabled=False, **ignored):
        blog_id = self.context.get("blog_id")

        try:
            await self.application.blogs.update_blog(self.gamespace, blog_id, name, schema, enabled)
        except BlogError:
            raise a.ActionError("Failed to update blog")

        raise a.Redirect("blog_settings", message="Blog has been updated", blog_id=blog_id)

    async def delete(self):
        blog_id = self.context.get("blog_id")

        try:
            await self.application.blogs.delete_blog(self.gamespace, blog_id)
        except BlogError:
            raise a.ActionError("Failed to delete blog")

        raise a.Redirect("index", message="Blog has been deleted")

    async def get(self, blog_id):
        try:
            blog = await self.application.blogs.get_blog(self.gamespace, blog_id)
        except BlogNotFound:
            raise a.ActionError("No such blog")

        return {
            "name": blog.name,
            "schema": blog.schema,
            "enabled": "true" if blog.enabled else "false"
        }

    @validate(name="str_name", schema="load_json_dict", enabled="bool")
    async def create(self, name, schema, enabled):
        try:
            blog_id = await self.application.blogs.create_blog(self.gamespace, name, schema, enabled)
        except BlogError as e:
            raise a.ActionError(e.args[0])

        raise a.Redirect("blog", message="New blog has been created", blog_id=blog_id)

    def access_scopes(self):
        return ["blog_admin"]


class NewBlogEntryAdminController(a.AdminController):
    def render(self, data):
        new_entry = self.context.get("clone", None) is None
        return [
            a.breadcrumbs([
                a.link("blog", data["blog_name"], blog_id=self.context.get("blog_id"))
            ], "New Blog Entry" if new_entry else "Clone Blog Entry"),
            a.form("Blog Entry", fields={
                "data": a.field("Entry", "dorn", "primary", schema=data["blog_schema"], order=1),
                "enabled": a.field("Enabled", "switch", "primary", order=2)
            }, methods={
                "create": a.method("Create" if new_entry else "Clone", "primary")
            }, data=data),
            a.links("Navigate", [
                a.link("blog", "Go back", icon="chevron-left", blog_id=self.context.get("blog_id")),
                a.link("blog_settings", "Edit Blog Settings", icon="cogs", blog_id=self.context.get("blog_id"))
            ])
        ]

    async def get(self, blog_id, clone=None):
        try:
            blog = await self.application.blogs.get_blog(self.gamespace, blog_id)
        except BlogNotFound:
            raise a.ActionError("No such blog")

        if clone is None:
            enabled = "true"
            data = {}
        else:
            try:
                entry = await self.application.blogs.get_blog_entry(self.gamespace, blog_id, clone)
            except BlogNotFound:
                raise a.ActionError("No such blog")
            else:
                enabled = "true" if entry.enabled else "false"
                data = entry.data

        return {
            "blog_name": blog.name,
            "blog_schema": blog.schema,
            "enabled": enabled,
            "data": data
        }

    @validate(data="load_json_dict", enabled="bool")
    async def create(self, data, enabled=False, **ignored):
        blog_id = self.context.get("blog_id")
        try:
            entry_id = await self.application.blogs.create_blog_entry(self.gamespace, blog_id, data, enabled)
        except BlogError as e:
            raise a.ActionError(e.args[0])

        raise a.Redirect("blog", message="New blog entry has been created", blog_id=blog_id)

    def access_scopes(self):
        return ["blog_admin"]


class BlogEntryAdminController(a.AdminController):
    def render(self, data):
        return [
            a.breadcrumbs([
                a.link("blog", data["blog_name"], blog_id=self.context.get("blog_id"))
            ], "Blog Entry #{0}".format(self.context.get("entry_id"))),
            a.form("Blog Entry", fields={
                "data": a.field("Entry", "dorn", "primary", schema=data["blog_schema"], order=1),
                "enabled": a.field("Enabled", "switch", "primary", order=2)
            }, methods={
                "update": a.method("Update", "primary"),
                "delete": a.method("Delete", "danger")
            }, data=data),
            a.links("Navigate", [
                a.link("blog", "Go back", icon="chevron-left", blog_id=self.context.get("blog_id")),
                a.link("new_blog_entry", "Clone This Entry", icon="clone", blog_id=self.context.get("blog_id"),
                       clone=self.context.get("entry_id")),
                a.link("blog_settings", "Edit Blog Settings", icon="cogs", blog_id=self.context.get("blog_id"))
            ])
        ]

    async def get(self, blog_id, entry_id):
        try:
            blog = await self.application.blogs.get_blog(self.gamespace, blog_id)
        except BlogNotFound:
            raise a.ActionError("No such blog")

        try:
            entry = await self.application.blogs.get_blog_entry(self.gamespace, blog_id, entry_id)
        except BlogNotFound:
            raise a.ActionError("No such blog")

        return {
            "blog_name": blog.name,
            "blog_schema": blog.schema,
            "data": entry.data,
            "enabled": "true" if entry.enabled else "false",
        }

    @validate()
    async def delete(self, **ignored):
        blog_id = self.context.get("blog_id")
        entry_id = self.context.get("entry_id")

        try:
            await self.application.blogs.get_blog(self.gamespace, blog_id)
        except BlogNotFound:
            raise a.ActionError("No such blog")

        try:
            await self.application.blogs.delete_blog_entry(self.gamespace, blog_id, entry_id)
        except BlogError as e:
            raise a.ActionError(e.args[0])

        raise a.Redirect("blog", message="Blog entry has been deleted", blog_id=blog_id)

    @validate(data="load_json_dict", enabled="bool")
    async def update(self, data, enabled=False, **ignored):
        blog_id = self.context.get("blog_id")
        entry_id = self.context.get("entry_id")

        try:
            await self.application.blogs.get_blog(self.gamespace, blog_id)
        except BlogNotFound:
            raise a.ActionError("No such blog")

        try:
            await self.application.blogs.update_blog_entry(self.gamespace, blog_id, entry_id, data, enabled)
        except BlogError as e:
            raise a.ActionError(e.args[0])

        raise a.Redirect("blog", message="Blog entry has been updated", blog_id=blog_id)

    def access_scopes(self):
        return ["blog_admin"]




from setuptools import setup, find_namespace_packages

DEPENDENCIES = [
    "anthill-common>=0.2.4"
]

setup(
    name='anthill-blog',
    package_data={
      "anthill.event": ["anthill/blog/sql", "anthill/blog/static"]
    },
    version='0.2',
    description='A service to deliver news and patch notes feed to the users inside the game',
    author='desertkun',
    license='MIT',
    author_email='desertkun@gmail.com',
    url='https://github.com/anthill-platform/anthill-blog',
    namespace_packages=["anthill"],
    include_package_data=True,
    packages=find_namespace_packages(include=["anthill.*"]),
    zip_safe=False,
    install_requires=DEPENDENCIES
)




from tornado.web import HTTPError

from anthill.common.handler import AuthenticatedHandler
from anthill.common.access import scoped, AccessToken
from anthill.common import to_int

from . model.blog import BlogNotFound, BlogError


class BlogsHandler(AuthenticatedHandler):
    @scoped(["blog"])
    async def get(self):

        blogs = self.application.blogs
        gamespace_id = self.token.get(AccessToken.GAMESPACE)

        try:
            blogs_list = await blogs.list_blogs(gamespace_id)
        except BlogError as e:
            raise HTTPError(e.code, e.args[0])

        self.dumps({
            "blogs": [
                blog.dump()
                for blog in blogs_list
            ]
        })


class BlogHandler(AuthenticatedHandler):
    @scoped(["blog"])
    async def get(self, blog_name):

        blogs = self.application.blogs
        gamespace_id = self.token.get(AccessToken.GAMESPACE)

        limit = to_int(self.get_argument("limit", 20))

        try:
            blog = await blogs.find_blog(gamespace_id, blog_name)
        except BlogNotFound:
            raise HTTPError(404, "No such blog")
        except BlogError as e:
            raise HTTPError(e.code, e.args[0])

        try:
            blog_entries = await blogs.list_enabled_blog_entries(gamespace_id, blog.blog_id, limit=limit)
        except BlogNotFound:
            raise HTTPError(404, "No such blog")
        except BlogError as e:
            raise HTTPError(e.code, e.args[0])

        self.dumps({
            "entries": [
                entry.dump()
                for entry in blog_entries
            ]
        })


class InternalHandler(object):
    def __init__(self, application):
        self.application = application




from anthill.common.options import options

from anthill.common.server import Server, init, start
from anthill.common.access import AccessToken, public
from anthill.common.keyvalue import KeyValueStorage
from anthill.common.database import Database

from . import handler as h
from . import admin
from . import options as _opts
from . model.blog import BlogsModel


class BlogServer(Server):
    # noinspection PyShadowingNames
    def __init__(self):
        super(BlogServer, self).__init__()

        self.db = Database(
            host=options.db_host,
            database=options.db_name,
            user=options.db_username,
            password=options.db_password)

        self.blogs = BlogsModel(self.db, self)

        self.cache = KeyValueStorage(
            host=options.cache_host,
            port=options.cache_port,
            db=options.cache_db,
            max_connections=options.cache_max_connections)

    def get_admin(self):
        return {
            "index": admin.RootAdminController,
            "new_blog": admin.NewBlogAdminController,
            "blog": admin.BlogAdminController,
            "blog_settings": admin.BlogSettingsAdminController,
            "new_blog_entry": admin.NewBlogEntryAdminController,
            "blog_entry": admin.BlogEntryAdminController
        }

    def get_models(self):
        return [self.blogs]

    def get_internal_handler(self):
        return h.InternalHandler(self)

    def get_metadata(self):
        return {
            "title": "Blogs",
            "description": "A service to deliver news and patch notes feed to the users inside the game",
            "icon": "rss"
        }

    def get_handlers(self):
        return [
            (r"/blogs", h.BlogsHandler),
            (r"/blog/(.*)", h.BlogHandler)
        ]


if __name__ == "__main__":
    stt = init()
    AccessToken.init([public()])
    start(BlogServer)




from anthill.common.options import define

# Main

define("host",
       default="http://localhost:9518",
       help="Public hostname of this service",
       type=str)

define("listen",
       default="port:9518",
       help="Socket to listen. Could be a port number (port:N), or a unix domain socket (unix:PATH)",
       type=str)

define("name",
       default="blog",
       help="Service short name. User to discover by discovery service.",
       type=str)

# MySQL database

define("db_host",
       default="127.0.0.1",
       type=str,
       help="MySQL database location")

define("db_username",
       default="root",
       type=str,
       help="MySQL account username")

define("db_password",
       default="",
       type=str,
       help="MySQL account password")

define("db_name",
       default="dev_blog",
       type=str,
       help="MySQL database name")

# Regular cache

define("cache_host",
       default="127.0.0.1",
       help="Location of a regular cache (redis).",
       group="cache",
       type=str)

define("cache_port",
       default=6379,
       help="Port of regular cache (redis).",
       group="cache",
       type=int)

define("cache_db",
       default=4,
       help="Database of regular cache (redis).",
       group="cache",
       type=int)

define("cache_max_connections",
       default=500,
       help="Maximum connections to the regular cache (connection pool).",
       group="cache",
       type=int)

# Events

define("schedule_update",
       default=300,
       type=int,
       help="A period in seconds to update a scheduled actions")
